///////////////////////////////////////////////////////////////////////
/////// FICHIER			: GBNikonRemote_10.c
/////// DESCRIPTION		: Nikon D70 IR remote - Soft v1.0
/////// PCB				: GBNikonRemote_v1
/////// AUTEUR			: GB
/////// DATE			: 02/2010
///////////////////////////////////////////////////////////////////////
/////// v1.0 :
///////		- Program created
///////
///////////////////////////////////////////////////////////////////////

/////////////////////////////// Configuration /////////////////////////
#include <16F628A.h>

#fuses NOPROTECT,PUT,NOWDT,HS
// Voir CCS C / View / Valid fuses

#use delay(clock=12000000)

/////////////////////////////////////////////////////////////
///////////////////        I/O          /////////////////////
/////////////////////////////////////////////////////////////
#define	OUTPUT_IRLed			PIN_B3
#define	INPUT_PushButton		(!input(PIN_A1))
#define	INPUT_ExternalTrigger	(!input(PIN_A0))
#define	Show_Mode(X)			output_b(X)

/////////////////////////////////////////////////////////////
///////////////////     Variables       /////////////////////
/////////////////////////////////////////////////////////////
#define PERIODE_TIMER0	88			// 40.3 us

int			loop_counter;
long		StillToWait;
#define		T 10		//	10*40.3 us = 403 us
int			FlashBeat_Timer;
int			FlashBeat_Counter;
short		LockFlashBeat;

int			RepeatMode;
#define		MODE_SingleShot	1
#define		MODE_Repeat_1s	2
#define		MODE_Repeat_5s	3
#define		MODE_Repeat_20s	4
#define		MODE_Repeat_1min	5
#define		MODE_Repeat_5min	6

/////////////////////////////////////////////////////////////
///////////////////     Fonctions   /////////////////////////
/////////////////////////////////////////////////////////////

/////////////////////////////////////////////////////////////
void IRLedPulseOFF()
{
	set_pwm1_duty(0);
}

/////////////////////////////////////////////////////////////
void IRLedPulseON()
{
	set_pwm1_duty(40);	// ~= 77/2 => 50%
}

/////////////////////////////////////////////////////////////
void System_Init()
{
	// Direction des ports
	Set_Tris_A(0b11111111);		// Direction PORTA (1=entre)
	Set_Tris_B(0b00000000);		// Direction PORTB

	// Init E/S
	output_low(OUTPUT_IRLed);
	Show_Mode(0);
	
	// Init variables
	StillToWait = 0;
	FlashBeat_Counter = 0;
	LockFlashBeat = 1;
	
	// Load RepeatMode from EEPROM
	RepeatMode = READ_EEPROM(0);
	if ((RepeatMode > MODE_Repeat_5min) || (RepeatMode == 0)) RepeatMode = MODE_SingleShot;

	//Timer 0 (40.3 us)
	setup_timer_0(RTCC_INTERNAL | RTCC_DIV_1);
	set_timer0(256-PERIODE_TIMER0+2);	// +2 to take into account the 2 cycle freeze after Timer0 write
	enable_interrupts(INT_TIMER0);

	//Timer 2 (PWM)			38.4 kHz IR signal frequency
	setup_ccp1(CCP_PWM);
	setup_timer_2(T2_DIV_BY_1, 77, 1);
	IRLedPulseOFF();

	enable_interrupts(GLOBAL);
}	

/////////////////////////////////////////////////////////////
void Delay_40p3us(long Duration)	// This function ensures exact duration (not like delay_us())
									// Duration is multiples of 40.3 us
{
	StillToWait = Duration;
	while (StillToWait);
}

/////////////////////////////////////////////////////////////
void SEND_IR_SingleFrame()
{
	// Show all LEDs
	LockFlashBeat = 1;
	Show_Mode(7);
	
	// Send frame
	IRLedPulseON();
	Delay_40p3us(5*T);
	IRLedPulseOFF();
	Delay_40p3us(70*T);
	IRLedPulseON();
	Delay_40p3us(T);
	IRLedPulseOFF();
	Delay_40p3us(4*T);
	IRLedPulseON();
	Delay_40p3us(T);
	IRLedPulseOFF();
	Delay_40p3us(9*T);
	IRLedPulseON();
	Delay_40p3us(T);
	IRLedPulseOFF();
	Delay_40p3us(158*T);
	
	// Switch IR LED Off
	output_low(OUTPUT_IRLed);
	
	// Hide LEDs
	Show_Mode(0);
	LockFlashBeat = 0;
}

void SEND_IR_Frame()	// Envoie 2 fois la trame
{
	SEND_IR_SingleFrame();
	SEND_IR_SingleFrame();
}

/////////////////////////////////////////////////////////////
#INT_TIMER0
void Timer_40p3us()		// 40.3us
{
	set_timer0(256-PERIODE_TIMER0+2);	// +2 to take into account the 2 cycle freeze after Timer0 write
	
	// Manage waiting timer
	if (StillToWait) StillToWait--;
	
	if ((--FlashBeat_Timer == 0) && (!LockFlashBeat))	// Every 256*40.3us = 10 ms
	{
		if (--FlashBeat_Counter == 0)		// Every 200 * 10ms = 2 s
		{
			Show_Mode(1);
			FlashBeat_Counter = 200;
		}
		else Show_Mode(0);
	}
}

/////////////////////////////////////////////////////////////
///////////////////        Main     /////////////////////////
/////////////////////////////////////////////////////////////
void main()
{
	System_Init();
	
	// Push button is pressed from the start => SETUP Mode
	if (INPUT_PushButton)
	{
		Show_Mode(RepeatMode);
		// Wait for button to be released
		while (INPUT_PushButton);
		
		// Loop and manage button
		while (1)
		{
			if (INPUT_PushButton)
			{
				// Change mode
				RepeatMode++;
				if (RepeatMode > MODE_Repeat_5min) RepeatMode = MODE_SingleShot;
				Show_Mode(RepeatMode);
				
				// Save in EEPROM
				WRITE_EEPROM(0, RepeatMode);
				
				// Wait for button to be released
				while (INPUT_PushButton);
			}
		}		
	}

	// ... else NORMAL mode
	Show_Mode(RepeatMode);
	delay_ms(1000);
	LockFlashBeat = 0;
	Show_Mode(0);
	
	while(1)
	{
		if (INPUT_PushButton || INPUT_ExternalTrigger)
		{
			switch (RepeatMode)
			{
				case MODE_SingleShot :
					// Send IR frame 2 times
					SEND_IR_Frame();
					while (INPUT_PushButton || INPUT_ExternalTrigger);
					break;
					
				default :	// repeat modes
					while (1)
					{
						SEND_IR_Frame();
						
						// Wait N seconds
						switch (RepeatMode)
						{
							case MODE_Repeat_1s :
								delay_ms(400);	// Longer in reality due to interrupts
								break;
								
							case MODE_Repeat_5s :
								delay_ms(2400);
								break;
								
							case MODE_Repeat_20s :
								delay_ms(10000);
								break;
								
							case MODE_Repeat_1min :
								delay_ms(30000);
								break;
								
							case MODE_Repeat_5min :
								for(loop_counter = 0; loop_counter < 5; loop_counter++)	delay_ms(30000);
								break;
						}
					}
					break;
			}
		}
	}
}